#version 150 core

in vec2 textureCoord;

uniform samplerCube depthCubemapBuffer;
uniform sampler2D   depthBuffer;

uniform mat4 viewProjectionInversedMatrix;

uniform bool homogeneousReversedDepth;
uniform vec3 wsLightPosition;
uniform int  lightType;

out float mask;

vec3 ssToNDC( in vec3 ssPosition ) {
  vec3 ndcPosition = vec3( ssPosition.xy * 2.0f - 1.0f, ssPosition.z );
  if ( !homogeneousReversedDepth ) {
    // если глубина должна быть в диапазоне [-1, 1]
    ndcPosition.z = ndcPosition.z * 2.0f - 1.0f;
  }
  return ndcPosition;
}

// вычисляем позицию в мировых координатах на основе однородных координат [0, 1] экрана и глубины
vec3 ssToWSPosition( in vec3 ssPosition ) {
  vec4 wsPosition = viewProjectionInversedMatrix * vec4( ssToNDC( ssPosition ), 1.0f );
  return wsPosition.xyz / wsPosition.w;
}

void main() {
  ivec2 bufferSize;
  if ( lightType == 1 ) {
    // cubemap
    bufferSize = textureSize( depthCubemapBuffer, 0 );
  } else {
    bufferSize = textureSize( depthBuffer, 0 );
  }

  float offset = 4.0 / bufferSize.x;

  //                            0   1   2   3  4   5   6   7   8
  float kernel[ 9 ] = float[]( -1, -1, -1, -1, 8, -1, -1, -1, -1 );
  // для граничных положений меняем ядро, чтобы избавиться от рамок по границе изображения
  if ( textureCoord.x <= offset ) {
    kernel[ 0 ] = 0;
    kernel[ 3 ] = 0;
    kernel[ 6 ] = 0;
  }
  if ( textureCoord.y <= offset ) {
    kernel[ 6 ] = 0;
    kernel[ 7 ] = 0;
    kernel[ 8 ] = 0;
  }
  if ( textureCoord.x + offset >= 1.0f ) {
    kernel[ 2 ] = 0;
    kernel[ 5 ] = 0;
    kernel[ 8 ] = 0;
  }
  if ( textureCoord.y + offset >= 1.0f ) {
    kernel[ 0 ] = 0;
    kernel[ 1 ] = 0;
    kernel[ 2 ] = 0;
  }
  int sum = 0;
  for ( int i = 0; i < 9; ++i ) {
    if ( kernel[ i ] == 0 ) {
      ++sum;
    }
  }
  kernel[ 4 ] = 8 - sum;

  vec2 offsetList[ 9 ] = vec2[](  //
    vec2( -offset, offset ),      // top-left
    vec2( 0.0f, offset ),         // top-center
    vec2( offset, offset ),       // top-right
    vec2( -offset, 0.0f ),        // center-left
    vec2( 0.0f, 0.0f ),           // center-center
    vec2( offset, 0.0f ),         // center-right
    vec2( -offset, -offset ),     // bottom-left
    vec2( 0.0f, -offset ),        // bottom-center
    vec2( offset, -offset )       // bottom-right
  );

  float sampleTex[ 9 ];
  if ( lightType == 1 ) {
    for ( int i = 0; i < 9; ++i ) {
      vec3 sampleDirection = wsLightPosition - ssToWSPosition( vec3( textureCoord + offsetList[ i ], 1.0f ) );
      sampleTex[ i ]       = texture( depthCubemapBuffer, -sampleDirection ).x;
    }
  } else {
    for ( int i = 0; i < 9; ++i ) {
      sampleTex[ i ] = texture( depthBuffer, textureCoord + offsetList[ i ] ).x;
    }
  }

  float color = 0.0f;
  for ( int i = 0; i < 9; ++i ) {
    color += sampleTex[ i ] * kernel[ i ];
  }

  mask = color > 0.00025f ? 1.0f : 0.0f;
}
