#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

layout( location = 0 ) in vec3 inVertex;
layout( location = 1 ) in vec3 inTextureCoord;
layout( location = 2 ) in vec3 inNormal;
layout( location = 3 ) in vec3 inTangent;
layout( location = 4 ) in vec3 inBoneId;
layout( location = 5 ) in vec3 inWeight;

uniform vec3 wsLightDir;
uniform mat4 viewProjectionMatrix;
uniform mat4 modelMatrix;

const int    maxBones   = 64;
const int    maxWeights = 2;
uniform mat4 boneMatrices[ maxBones ];

out vec2 textureCoord;
out vec3 lightDir;
out mat3 tbnMatrix;

void main() {
  vec4 resultPosition = vec4( 0.0f );
  vec4 resultNormal   = vec4( 0.0f );
  vec4 resultTangent  = vec4( 0.0f );

  for ( int i = 0; i < maxWeights; ++i ) {
    mat4 boneTransform = boneMatrices[ int( inBoneId[ i ] ) ];

    vec4 localPosition = boneTransform * vec4( inVertex, 1.0f );
    resultPosition += localPosition * inWeight[ i ];

    vec4 worldNormal = boneTransform * vec4( inNormal, 0.0f );
    resultNormal += worldNormal * inWeight[ i ];

    vec4 worldTangent = boneTransform * vec4( inTangent, 0.0f );
    resultTangent += worldTangent * inWeight[ i ];
  }

  gl_Position = viewProjectionMatrix * modelMatrix * resultPosition;

  textureCoord = inTextureCoord.xy;
  lightDir     = wsLightDir;

  // создаем матрицу перевода направления в координаты пространства мира
  mat3 normalMatrix = mat3( modelMatrix );
  vec3 wsNormal     = normalize( normalMatrix * resultNormal.xyz );
  vec3 wsTangent    = normalize( normalMatrix * resultTangent.xyz );
  vec3 wsBitangent  = normalize( cross( wsNormal, wsTangent ) );
  tbnMatrix         = mat3( wsTangent, wsBitangent, wsNormal );
}
