#version 110

varying vec2 textureCoord;
varying vec3 lightDir;
varying mat3 tbnMatrix;

uniform sampler2D textureAlbedo;
uniform sampler2D textureNormal;
uniform vec3      screenGamma;
uniform float     exposureLevel;
uniform float     alphaTestThreshold;
uniform bool      shouldDrawLights;
uniform bool      isColor;
uniform vec4      color;
uniform float     opacity;
uniform float     selfIllumination;

const vec3  lightColor = vec3( 1.0, 1.0, 1.0 );
const float ambient    = 0.05;

vec3 getNormal( in vec2 texCoord ) {
  // переводим из tangent space в NDC [-1.0, 1.0]
  vec3 ndcNormal = texture2D( textureNormal, texCoord ).xyz * 2.0 - 1.0;
  // переводим в мировое пространство
  return normalize( tbnMatrix * ndcNormal );
}

void main() {
  vec4 colorLinear;
  if ( isColor ) {
    colorLinear = color;
  } else {
    colorLinear = texture2D( textureAlbedo, textureCoord );
  }

  float alpha = min( colorLinear.a, opacity );
  if ( alpha < alphaTestThreshold ) {
    discard;
  }
  vec3 finalColor = colorLinear.rgb;
  if ( shouldDrawLights ) {
    float diffuse = max( dot( getNormal( textureCoord ), lightDir ), 0.0 );
    vec3  light   = max( ( diffuse + ambient ), selfIllumination ) * lightColor;
    finalColor    = light * colorLinear.rgb;
  }

  vec3 mapped  = vec3( 1.0 ) - exp( -finalColor * exposureLevel );
  gl_FragColor = vec4( pow( mapped, screenGamma ), alpha );
}
