#version 110

varying vec2 textureCoord;

uniform sampler2D albedo;
uniform vec3      screenGamma;
uniform bool      isColor;
uniform vec4      color;
uniform bool      isFont;
uniform float     fontSmoothing;
uniform bool      hasOutline;
uniform float     outlineWidth;
uniform vec4      outlineColor;

const float exposureLevel = 1.0f;
const float symbolWidth   = 0.5f;

void main() {
  vec4 resultColor;
  if ( isColor && isFont ) {
    resultColor        = color;
    float distance     = texture2D( albedo, textureCoord ).a;
    float smoothFactor = smoothstep( symbolWidth - fontSmoothing, symbolWidth + fontSmoothing, distance );
    if ( hasOutline ) {
      resultColor  = mix( outlineColor, resultColor, smoothFactor );
      smoothFactor = min( resultColor.a, smoothstep( outlineWidth - fontSmoothing, outlineWidth + fontSmoothing, distance ) );
    }
    resultColor = vec4( resultColor.rgb, smoothFactor );
  } else if ( isColor ) {
    resultColor = color;
  } else {
    resultColor = texture2D( albedo, textureCoord );
  }

  // Exposure tone mapping
  vec3 mapped  = vec3( 1.0 ) - exp( -resultColor.rgb * exposureLevel );
  gl_FragColor = vec4( pow( mapped, screenGamma ), resultColor.a );
}
