#version 150 core
#extension GL_ARB_explicit_attrib_location : enable

layout( location = 0 ) in vec3 inVertex;
layout( location = 4 ) in vec3 inBoneId;
layout( location = 5 ) in vec3 inWeight;

uniform mat4 viewProjectionMatrix;
uniform mat4 modelMatrix;
uniform vec4 color;

uniform bool drawBoundingBoxFlag;

const int    maxBones   = 64;
const int    maxWeights = 2;
uniform mat4 boneMatrices[ maxBones ];

out vec4 drawColor;

void main() {
  vec4 resultPosition = vec4( 0.0f );

  if ( drawBoundingBoxFlag ) {
    resultPosition = vec4( inVertex, 1.0f );
  } else {
    for ( int i = 0; i < maxWeights; ++i ) {
      mat4 boneTransform = boneMatrices[ int( inBoneId[ i ] ) ];
      vec4 localPosition = boneTransform * vec4( inVertex, 1.0f );
      resultPosition += localPosition * inWeight[ i ];
    }
  }
  gl_Position = viewProjectionMatrix * modelMatrix * resultPosition;

  drawColor = color;
}
